@@ -22,29 +22,39 @@
 
 import asyncio
 import random
-from functools import lru_cache
-from ipaddress import IPv4Address, IPv6Address, _BaseAddress, ip_address
+import sys
+from ipaddress import IPv4Address, IPv6Address, _BaseAddress
 from typing import TYPE_CHECKING, Dict, List, Optional, Set, Union, cast
 
+from .._cache import DNSCache
 from .._dns import (
     DNSAddress,
     DNSNsec,
     DNSPointer,
+    DNSQuestion,
     DNSQuestionType,
     DNSRecord,
     DNSService,
     DNSText,
 )
 from .._exceptions import BadTypeInNameException
+from .._history import QuestionHistory
 from .._logger import log
 from .._protocol.outgoing import DNSOutgoing
-from .._updates import RecordUpdate, RecordUpdateListener
+from .._record_update import RecordUpdate
+from .._updates import RecordUpdateListener
 from .._utils.asyncio import (
     _resolve_all_futures_to_none,
     get_running_loop,
     run_coro_with_timeout,
     wait_for_future_set_or_timeout,
 )
+from .._utils.ipaddress import (
+    cached_ip_addresses,
+    get_ip_address_object_from_record,
+    ip_bytes_and_scope_to_address,
+    str_without_scope_id,
+)
 from .._utils.name import service_type_name
 from .._utils.net import IPVersion, _encode_address
 from .._utils.time import current_time_millis
@@ -54,6 +64,7 @@ from ..const import (
     _CLASS_IN_UNIQUE,
     _DNS_HOST_TTL,
     _DNS_OTHER_TTL,
+    _DUPLICATE_QUESTION_INTERVAL,
     _FLAGS_QR_QUERY,
     _LISTENER_TIME,
     _MDNS_PORT,
@@ -65,6 +76,8 @@ from ..const import (
     _TYPE_TXT,
 )
 
+IPADDRESS_SUPPORTS_SCOPE_ID = sys.version_info >= (3, 9, 0)
+
 _IPVersion_All_value = IPVersion.All.value
 _IPVersion_V4Only_value = IPVersion.V4Only.value
 # https://datatracker.ietf.org/doc/html/rfc6762#section-5.2
@@ -77,6 +90,16 @@ _IPVersion_V4Only_value = IPVersion.V4Only.value
 # the A/AAAA/SRV records for a host.
 _AVOID_SYNC_DELAY_RANDOM_INTERVAL = (20, 120)
 
+bytes_ = bytes
+float_ = float
+int_ = int
+str_ = str
+
+QU_QUESTION = DNSQuestionType.QU
+QM_QUESTION = DNSQuestionType.QM
+
+randint = random.randint
+
 if TYPE_CHECKING:
     from .._core import Zeroconf
 
@@ -91,9 +114,6 @@ def instance_name_from_service_info(info: "ServiceInfo", strict: bool = True) ->
     return info.name[: -len(service_name) - 1]
 
 
-_cached_ip_addresses = lru_cache(maxsize=256)(ip_address)
-
-
 class ServiceInfo(RecordUpdateListener):
     """Service information.
 
@@ -129,10 +149,16 @@ class ServiceInfo(RecordUpdateListener):
         "server",
         "server_key",
         "_properties",
+        "_decoded_properties",
         "host_ttl",
         "other_ttl",
         "interface_index",
         "_new_records_futures",
+        "_dns_pointer_cache",
+        "_dns_service_cache",
+        "_dns_text_cache",
+        "_dns_address_cache",
+        "_get_address_and_nsec_records_cache",
     )
 
     def __init__(
@@ -156,6 +182,7 @@ class ServiceInfo(RecordUpdateListener):
             raise TypeError("addresses and parsed_addresses cannot be provided together")
         if not type_.endswith(service_type_name(name, strict=False)):
             raise BadTypeInNameException
+        self.interface_index = interface_index
         self.text = b''
         self.type = type_
         self._name = name
@@ -171,15 +198,20 @@ class ServiceInfo(RecordUpdateListener):
         self.priority = priority
         self.server = server if server else None
         self.server_key = server.lower() if server else None
-        self._properties: Optional[Dict[Union[str, bytes], Optional[Union[str, bytes]]]] = None
+        self._properties: Optional[Dict[bytes, Optional[bytes]]] = None
+        self._decoded_properties: Optional[Dict[str, Optional[str]]] = None
         if isinstance(properties, bytes):
             self._set_text(properties)
         else:
             self._set_properties(properties)
         self.host_ttl = host_ttl
         self.other_ttl = other_ttl
-        self.interface_index = interface_index
-        self._new_records_futures: Set[asyncio.Future] = set()
+        self._new_records_futures: Optional[Set[asyncio.Future]] = None
+        self._dns_address_cache: Optional[List[DNSAddress]] = None
+        self._dns_pointer_cache: Optional[DNSPointer] = None
+        self._dns_service_cache: Optional[DNSService] = None
+        self._dns_text_cache: Optional[DNSText] = None
+        self._get_address_and_nsec_records_cache: Optional[Set[DNSRecord]] = None
 
     @property
     def name(self) -> str:
@@ -191,6 +223,9 @@ class ServiceInfo(RecordUpdateListener):
         """Replace the the name and reset the key."""
         self._name = name
         self.key = name.lower()
+        self._dns_service_cache = None
+        self._dns_pointer_cache = None
+        self._dns_text_cache = None
 
     @property
     def addresses(self) -> List[bytes]:
@@ -210,40 +245,61 @@ class ServiceInfo(RecordUpdateListener):
         """
         self._ipv4_addresses.clear()
         self._ipv6_addresses.clear()
+        self._dns_address_cache = None
+        self._get_address_and_nsec_records_cache = None
 
         for address in value:
-            try:
-                addr = _cached_ip_addresses(address)
-            except ValueError:
+            if IPADDRESS_SUPPORTS_SCOPE_ID and len(address) == 16 and self.interface_index is not None:
+                addr = ip_bytes_and_scope_to_address(address, self.interface_index)
+            else:
+                addr = cached_ip_addresses(address)
+            if addr is None:
                 raise TypeError(
                     "Addresses must either be IPv4 or IPv6 strings, bytes, or integers;"
                     f" got {address!r}. Hint: convert string addresses with socket.inet_pton"
                 )
             if addr.version == 4:
+                if TYPE_CHECKING:
+                    assert isinstance(addr, IPv4Address)
                 self._ipv4_addresses.append(addr)
             else:
+                if TYPE_CHECKING:
+                    assert isinstance(addr, IPv6Address)
                 self._ipv6_addresses.append(addr)
 
     @property
-    def properties(self) -> Dict[Union[str, bytes], Optional[Union[str, bytes]]]:
-        """If properties were set in the constructor this property returns the original dictionary
-        of type `Dict[Union[bytes, str], Any]`.
-
-        If properties are coming from the network, after decoding a TXT record, the keys are always
-        bytes and the values are either bytes, if there was a value, even empty, or `None`, if there
-        was none. No further decoding is attempted. The type returned is `Dict[bytes, Optional[bytes]]`.
-        """
+    def properties(self) -> Dict[bytes, Optional[bytes]]:
+        """Return properties as bytes."""
         if self._properties is None:
             self._unpack_text_into_properties()
         if TYPE_CHECKING:
             assert self._properties is not None
         return self._properties
 
-    async def async_wait(self, timeout: float) -> None:
+    @property
+    def decoded_properties(self) -> Dict[str, Optional[str]]:
+        """Return properties as strings."""
+        if self._decoded_properties is None:
+            self._generate_decoded_properties()
+        if TYPE_CHECKING:
+            assert self._decoded_properties is not None
+        return self._decoded_properties
+
+    def async_clear_cache(self) -> None:
+        """Clear the cache for this service info."""
+        self._dns_address_cache = None
+        self._dns_pointer_cache = None
+        self._dns_service_cache = None
+        self._dns_text_cache = None
+        self._get_address_and_nsec_records_cache = None
+
+    async def async_wait(self, timeout: float, loop: Optional[asyncio.AbstractEventLoop] = None) -> None:
         """Calling task waits for a given number of milliseconds or until notified."""
-        loop = get_running_loop()
-        assert loop is not None
-        await wait_for_future_set_or_timeout(loop, self._new_records_futures, timeout)
+        if not self._new_records_futures:
+            self._new_records_futures = set()
+        await wait_for_future_set_or_timeout(
+            loop or asyncio.get_running_loop(), self._new_records_futures, timeout
+        )
 
     def addresses_by_version(self, version: IPVersion) -> List[bytes]:
         """List addresses matching IP version.
@@ -256,10 +312,9 @@ class ServiceInfo(RecordUpdateListener):
         """
         version_value = version.value
         if version_value == _IPVersion_All_value:
-            return [
-                *(addr.packed for addr in self._ipv4_addresses),
-                *(addr.packed for addr in self._ipv6_addresses),
-            ]
+            ip_v4_packed = [addr.packed for addr in self._ipv4_addresses]
+            ip_v6_packed = [addr.packed for addr in self._ipv6_addresses]
+            return [*ip_v4_packed, *ip_v6_packed]
         if version_value == _IPVersion_V4Only_value:
             return [addr.packed for addr in self._ipv4_addresses]
         return [addr.packed for addr in self._ipv6_addresses]
@@ -278,11 +333,11 @@ class ServiceInfo(RecordUpdateListener):
         return self._ip_addresses_by_version_value(version.value)
 
     def _ip_addresses_by_version_value(
-        self, version_value: int
-    ) -> Union[List[IPv4Address], List[IPv6Address], List[_BaseAddress]]:
+        self, version_value: int_
+    ) -> Union[List[IPv4Address], List[IPv6Address]]:
         """Backend for addresses_by_version that uses the raw value."""
         if version_value == _IPVersion_All_value:
-            return [*self._ipv4_addresses, *self._ipv6_addresses]
+            return [*self._ipv4_addresses, *self._ipv6_addresses]  # type: ignore[return-value]
         if version_value == _IPVersion_V4Only_value:
             return self._ipv4_addresses
         return self._ipv6_addresses
@@ -296,7 +351,7 @@ class ServiceInfo(RecordUpdateListener):
         This means the first address will always be the most recently added
         address of the given IP version.
         """
-        return [str(addr) for addr in self._ip_addresses_by_version_value(version.value)]
+        return [str_without_scope_id(addr) for addr in self._ip_addresses_by_version_value(version.value)]
 
     def parsed_scoped_addresses(self, version: IPVersion = IPVersion.All) -> List[str]:
         """Equivalent to parsed_addresses, with the exception that IPv6 Link-Local
@@ -308,30 +363,35 @@ class ServiceInfo(RecordUpdateListener):
         This means the first address will always be the most recently added
         address of the given IP version.
         """
-        if self.interface_index is None:
-            return self.parsed_addresses(version)
-        return [
-            f"{addr}%{self.interface_index}" if addr.version == 6 and addr.is_link_local else str(addr)
-            for addr in self._ip_addresses_by_version_value(version.value)
-        ]
+        return [str(addr) for addr in self._ip_addresses_by_version_value(version.value)]
 
     def _set_properties(self, properties: Dict[Union[str, bytes], Optional[Union[str, bytes]]]) -> None:
         """Sets properties and text of this info from a dictionary"""
-        self._properties = properties
         list_: List[bytes] = []
+        properties_contain_str = False
         result = b''
         for key, value in properties.items():
             if isinstance(key, str):
                 key = key.encode('utf-8')
+                properties_contain_str = True
 
             record = key
             if value is not None:
                 if not isinstance(value, bytes):
                     value = str(value).encode('utf-8')
+                    properties_contain_str = True
                 record += b'=' + value
             list_.append(record)
         for item in list_:
             result = b''.join((result, bytes((len(item),)), item))
+        if not properties_contain_str:
+            # If there are no str keys or values, we can use the properties
+            # as-is, without decoding them, otherwise calling
+            # self.properties will lazy decode them, which is expensive.
+            if TYPE_CHECKING:
+                self._properties = cast("Dict[bytes, Optional[bytes]]", properties)
+            else:
+                self._properties = properties
         self.text = result
 
     def _set_text(self, text: bytes) -> None:
@@ -341,54 +401,58 @@ class ServiceInfo(RecordUpdateListener):
         self.text = text
         # Clear the properties cache
         self._properties = None
+        self._decoded_properties = None
+
+    def _generate_decoded_properties(self) -> None:
+        """Generates decoded properties from the properties"""
+        self._decoded_properties = {
+            k.decode("ascii", "replace"): None if v is None else v.decode("utf-8", "replace")
+            for k, v in self.properties.items()
+        }
 
     def _unpack_text_into_properties(self) -> None:
         """Unpacks the text field into properties"""
         text = self.text
-        if not text:
+        end = len(text)
+        if end == 0:
             # Properties should be set atomically
             # in case another thread is reading them
             self._properties = {}
             return
 
         index = 0
-        pairs: List[bytes] = []
-        end = len(text)
+        properties: Dict[bytes, Optional[bytes]] = {}
         while index < end:
             length = text[index]
             index += 1
-            pairs.append(text[index : index + length])
+            key_value = text[index : index + length]
+            key_sep_value = key_value.partition(b'=')
+            key = key_sep_value[0]
+            if key not in properties:
+                properties[key] = key_sep_value[2] or None
             index += length
 
-        # Reverse the list so that the first item in the list
-        # is the last item in the text field. This is important
-        # to preserve backwards compatibility where the first
-        # key always wins if the key is seen multiple times.
-        pairs.reverse()
-        self._properties = {key: value or None for key, _, value in (pair.partition(b'=') for pair in pairs)}
+        self._properties = properties
 
     def get_name(self) -> str:
         """Name accessor"""
         return self._name[: len(self._name) - len(self.type) - 1]
 
     def _get_ip_addresses_from_cache_lifo(
-        self, zc: 'Zeroconf', now: float, type: int
+        self, zc: 'Zeroconf', now: float_, type: int_
     ) -> List[Union[IPv4Address, IPv6Address]]:
         """Set IPv6 addresses from the cache."""
         address_list: List[Union[IPv4Address, IPv6Address]] = []
         for record in self._get_address_records_from_cache_by_type(zc, type):
             if record.is_expired(now):
                 continue
-            try:
-                ip_addr = _cached_ip_addresses(record.address)
-            except ValueError:
-                continue
-            else:
+            ip_addr = get_ip_address_object_from_record(record)
+            if ip_addr is not None and ip_addr not in address_list:
                 address_list.append(ip_addr)
         address_list.reverse()  # Reverse to get LIFO order
         return address_list
 
-    def _set_ipv6_addresses_from_cache(self, zc: 'Zeroconf', now: float) -> None:
+    def _set_ipv6_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
         """Set IPv6 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv6_addresses = cast(
@@ -397,7 +461,7 @@ class ServiceInfo(RecordUpdateListener):
         else:
             self._ipv6_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
 
-    def _set_ipv4_addresses_from_cache(self, zc: 'Zeroconf', now: float) -> None:
+    def _set_ipv4_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
         """Set IPv4 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv4_addresses = cast(
@@ -406,7 +470,7 @@ class ServiceInfo(RecordUpdateListener):
         else:
             self._ipv4_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
 
-    def async_update_records(self, zc: 'Zeroconf', now: float, records: List[RecordUpdate]) -> None:
+    def async_update_records(self, zc: 'Zeroconf', now: float_, records: List[RecordUpdate]) -> None:
         """Updates service information from a DNS record.
 
         This method will be run in the event loop.
@@ -418,7 +482,7 @@ class ServiceInfo(RecordUpdateListener):
         if updated and new_records_futures:
             _resolve_all_futures_to_none(new_records_futures)
 
-    def _process_record_threadsafe(self, zc: 'Zeroconf', record: DNSRecord, now: float) -> bool:
+    def _process_record_threadsafe(self, zc: 'Zeroconf', record: DNSRecord, now: float_) -> bool:
         """Thread safe record updating.
 
         Returns True if a new record was added.
@@ -428,16 +492,22 @@ class ServiceInfo(RecordUpdateListener):
 
         record_key = record.key
         record_type = type(record)
-        if record_key == self.server_key and record_type is DNSAddress:
+        if record_type is DNSAddress and record_key == self.server_key:
+            dns_address_record = record
             if TYPE_CHECKING:
-                assert isinstance(record, DNSAddress)
-            try:
-                ip_addr = _cached_ip_addresses(record.address)
-            except ValueError as ex:
-                log.warning("Encountered invalid address while processing %s: %s", record, ex)
+                assert isinstance(dns_address_record, DNSAddress)
+            ip_addr = get_ip_address_object_from_record(dns_address_record)
+            if ip_addr is None:
+                log.warning(
+                    "Encountered invalid address while processing %s: %s",
+                    dns_address_record,
+                    dns_address_record.address,
+                )
                 return False
 
-            if type(ip_addr) is IPv4Address:
+            if ip_addr.version == 4:
+                if TYPE_CHECKING:
+                    assert isinstance(ip_addr, IPv4Address)
                 ipv4_addresses = self._ipv4_addresses
                 if ip_addr not in ipv4_addresses:
                     ipv4_addresses.insert(0, ip_addr)
@@ -448,6 +518,8 @@ class ServiceInfo(RecordUpdateListener):
 
                 return False
 
+            if TYPE_CHECKING:
+                assert isinstance(ip_addr, IPv6Address)
             ipv6_addresses = self._ipv6_addresses
             if ip_addr not in self._ipv6_addresses:
                 ipv6_addresses.insert(0, ip_addr)
@@ -462,22 +534,24 @@ class ServiceInfo(RecordUpdateListener):
             return False
 
         if record_type is DNSText:
+            dns_text_record = record
             if TYPE_CHECKING:
-                assert isinstance(record, DNSText)
-            self._set_text(record.text)
+                assert isinstance(dns_text_record, DNSText)
+            self._set_text(dns_text_record.text)
             return True
 
         if record_type is DNSService:
+            dns_service_record = record
             if TYPE_CHECKING:
-                assert isinstance(record, DNSService)
+                assert isinstance(dns_service_record, DNSService)
             old_server_key = self.server_key
-            self._name = record.name
-            self.key = record.key
-            self.server = record.server
-            self.server_key = record.server_key
-            self.port = record.port
-            self.weight = record.weight
-            self.priority = record.priority
+            self._name = dns_service_record.name
+            self.key = dns_service_record.key
+            self.server = dns_service_record.server
+            self.server_key = dns_service_record.server_key
+            self.port = dns_service_record.port
+            self.weight = dns_service_record.weight
+            self.priority = dns_service_record.priority
             if old_server_key != self.server_key:
                 self._set_ipv4_addresses_from_cache(zc, now)
                 self._set_ipv6_addresses_from_cache(zc, now)
@@ -489,42 +563,72 @@ class ServiceInfo(RecordUpdateListener):
         self,
         override_ttl: Optional[int] = None,
         version: IPVersion = IPVersion.All,
-        created: Optional[float] = None,
     ) -> List[DNSAddress]:
         """Return matching DNSAddress from ServiceInfo."""
+        return self._dns_addresses(override_ttl, version)
+
+    def _dns_addresses(
+        self,
+        override_ttl: Optional[int],
+        version: IPVersion,
+    ) -> List[DNSAddress]:
+        """Return matching DNSAddress from ServiceInfo."""
+        cacheable = version is IPVersion.All and override_ttl is None
+        if self._dns_address_cache is not None and cacheable:
+            return self._dns_address_cache
         name = self.server or self._name
         ttl = override_ttl if override_ttl is not None else self.host_ttl
         class_ = _CLASS_IN_UNIQUE
         version_value = version.value
-        return [
+        records = [
             DNSAddress(
                 name,
-                _TYPE_AAAA if type(ip_addr) is IPv6Address else _TYPE_A,
+                _TYPE_AAAA if ip_addr.version == 6 else _TYPE_A,
                 class_,
                 ttl,
                 ip_addr.packed,
-                created=created,
+                created=0.0,
             )
             for ip_addr in self._ip_addresses_by_version_value(version_value)
         ]
+        if cacheable:
+            self._dns_address_cache = records
+        return records
 
-    def dns_pointer(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSPointer:
+    def dns_pointer(self, override_ttl: Optional[int] = None) -> DNSPointer:
         """Return DNSPointer from ServiceInfo."""
-        return DNSPointer(
+        return self._dns_pointer(override_ttl)
+
+    def _dns_pointer(self, override_ttl: Optional[int]) -> DNSPointer:
+        """Return DNSPointer from ServiceInfo."""
+        cacheable = override_ttl is None
+        if self._dns_pointer_cache is not None and cacheable:
+            return self._dns_pointer_cache
+        record = DNSPointer(
             self.type,
             _TYPE_PTR,
             _CLASS_IN,
             override_ttl if override_ttl is not None else self.other_ttl,
             self._name,
-            created,
+            0.0,
         )
+        if cacheable:
+            self._dns_pointer_cache = record
+        return record
 
-    def dns_service(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSService:
+    def dns_service(self, override_ttl: Optional[int] = None) -> DNSService:
         """Return DNSService from ServiceInfo."""
+        return self._dns_service(override_ttl)
+
+    def _dns_service(self, override_ttl: Optional[int]) -> DNSService:
+        """Return DNSService from ServiceInfo."""
+        cacheable = override_ttl is None
+        if self._dns_service_cache is not None and cacheable:
+            return self._dns_service_cache
         port = self.port
         if TYPE_CHECKING:
             assert isinstance(port, int)
-        return DNSService(
+        record = DNSService(
             self._name,
             _TYPE_SRV,
             _CLASS_IN_UNIQUE,
@@ -533,23 +637,38 @@ class ServiceInfo(RecordUpdateListener):
             self.weight,
             port,
             self.server or self._name,
-            created,
+            0.0,
         )
+        if cacheable:
+            self._dns_service_cache = record
+        return record
 
-    def dns_text(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSText:
+    def dns_text(self, override_ttl: Optional[int] = None) -> DNSText:
         """Return DNSText from ServiceInfo."""
-        return DNSText(
+        return self._dns_text(override_ttl)
+
+    def _dns_text(self, override_ttl: Optional[int]) -> DNSText:
+        """Return DNSText from ServiceInfo."""
+        cacheable = override_ttl is None
+        if self._dns_text_cache is not None and cacheable:
+            return self._dns_text_cache
+        record = DNSText(
             self._name,
             _TYPE_TXT,
             _CLASS_IN_UNIQUE,
             override_ttl if override_ttl is not None else self.other_ttl,
             self.text,
-            created,
+            0.0,
         )
+        if cacheable:
+            self._dns_text_cache = record
+        return record
 
-    def dns_nsec(
-        self, missing_types: List[int], override_ttl: Optional[int] = None, created: Optional[float] = None
-    ) -> DNSNsec:
+    def dns_nsec(self, missing_types: List[int], override_ttl: Optional[int] = None) -> DNSNsec:
+        """Return DNSNsec from ServiceInfo."""
+        return self._dns_nsec(missing_types, override_ttl)
+
+    def _dns_nsec(self, missing_types: List[int], override_ttl: Optional[int]) -> DNSNsec:
         """Return DNSNsec from ServiceInfo."""
         return DNSNsec(
             self._name,
@@ -558,31 +677,39 @@ class ServiceInfo(RecordUpdateListener):
             override_ttl if override_ttl is not None else self.host_ttl,
             self._name,
             missing_types,
-            created,
+            0.0,
         )
 
-    def get_address_and_nsec_records(
-        self, override_ttl: Optional[int] = None, created: Optional[float] = None
-    ) -> Set[DNSRecord]:
+    def get_address_and_nsec_records(self, override_ttl: Optional[int] = None) -> Set[DNSRecord]:
         """Build a set of address records and NSEC records for non-present record types."""
+        return self._get_address_and_nsec_records(override_ttl)
+
+    def _get_address_and_nsec_records(self, override_ttl: Optional[int]) -> Set[DNSRecord]:
+        """Build a set of address records and NSEC records for non-present record types."""
+        cacheable = override_ttl is None
+        if self._get_address_and_nsec_records_cache is not None and cacheable:
+            return self._get_address_and_nsec_records_cache
         missing_types: Set[int] = _ADDRESS_RECORD_TYPES.copy()
         records: Set[DNSRecord] = set()
-        for dns_address in self.dns_addresses(override_ttl, IPVersion.All, created):
+        for dns_address in self._dns_addresses(override_ttl, IPVersion.All):
             missing_types.discard(dns_address.type)
             records.add(dns_address)
         if missing_types:
             assert self.server is not None, "Service server must be set for NSEC record."
-            records.add(self.dns_nsec(list(missing_types), override_ttl, created))
+            records.add(self._dns_nsec(list(missing_types), override_ttl))
+        if cacheable:
+            self._get_address_and_nsec_records_cache = records
         return records
 
-    def _get_address_records_from_cache_by_type(self, zc: 'Zeroconf', _type: int) -> List[DNSAddress]:
+    def _get_address_records_from_cache_by_type(self, zc: 'Zeroconf', _type: int_) -> List[DNSAddress]:
         """Get the addresses from the cache."""
         if self.server_key is None:
             return []
+        cache = zc.cache
         if TYPE_CHECKING:
-            records = cast("List[DNSAddress]", zc.cache.get_all_by_details(self.server_key, _type, _CLASS_IN))
+            records = cast("List[DNSAddress]", cache.get_all_by_details(self.server_key, _type, _CLASS_IN))
         else:
-            records = zc.cache.get_all_by_details(self.server_key, _type, _CLASS_IN)
+            records = cache.get_all_by_details(self.server_key, _type, _CLASS_IN)
         return records
 
     def set_server_if_missing(self) -> None:
@@ -594,28 +721,33 @@ class ServiceInfo(RecordUpdateListener):
             self.server = self._name
             self.server_key = self.key
 
-    def load_from_cache(self, zc: 'Zeroconf', now: Optional[float] = None) -> bool:
+    def load_from_cache(self, zc: 'Zeroconf', now: Optional[float_] = None) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
         """
-        if not now:
-            now = current_time_millis()
+        return self._load_from_cache(zc, now or current_time_millis())
+
+    def _load_from_cache(self, zc: 'Zeroconf', now: float_) -> bool:
+        """Populate the service info from the cache.
+
+        This method is designed to be threadsafe.
+        """
+        cache = zc.cache
         original_server_key = self.server_key
-        cached_srv_record = zc.cache.get_by_details(self._name, _TYPE_SRV, _CLASS_IN)
+        cached_srv_record = cache.get_by_details(self._name, _TYPE_SRV, _CLASS_IN)
         if cached_srv_record:
             self._process_record_threadsafe(zc, cached_srv_record, now)
-        cached_txt_record = zc.cache.get_by_details(self._name, _TYPE_TXT, _CLASS_IN)
+        cached_txt_record = cache.get_by_details(self._name, _TYPE_TXT, _CLASS_IN)
         if cached_txt_record:
             self._process_record_threadsafe(zc, cached_txt_record, now)
         if original_server_key == self.server_key:
             # If there is a srv which changes the server_key,
             # A and AAAA will already be loaded from the cache
             # and we do not want to do it twice
-            for record in [
-                *self._get_address_records_from_cache_by_type(zc, _TYPE_A),
-                *self._get_address_records_from_cache_by_type(zc, _TYPE_AAAA),
-            ]:
+            for record in self._get_address_records_from_cache_by_type(zc, _TYPE_A):
+                self._process_record_threadsafe(zc, record, now)
+            for record in self._get_address_records_from_cache_by_type(zc, _TYPE_AAAA):
                 self._process_record_threadsafe(zc, record, now)
         return self._is_complete
 
@@ -648,6 +780,12 @@ class ServiceInfo(RecordUpdateListener):
             )
         )
 
+    def _get_initial_delay(self) -> float_:
+        return _LISTENER_TIME
+
+    def _get_random_delay(self) -> int_:
+        return randint(*_AVOID_SYNC_DELAY_RANDOM_INTERVAL)
+
     async def async_request(
         self,
         zc: 'Zeroconf',
@@ -671,11 +809,14 @@ class ServiceInfo(RecordUpdateListener):
 
         now = current_time_millis()
 
-        if self.load_from_cache(zc, now):
+        if self._load_from_cache(zc, now):
             return True
 
+        if TYPE_CHECKING:
+            assert zc.loop is not None
+
         first_request = True
-        delay = _LISTENER_TIME
+        delay = self._get_initial_delay()
         next_ = now
         last = now + timeout
         try:
@@ -684,45 +825,86 @@ class ServiceInfo(RecordUpdateListener):
                 if last <= now:
                     return False
                 if next_ <= now:
-                    out = self.generate_request_query(
-                        zc, now, question_type or DNSQuestionType.QU if first_request else DNSQuestionType.QM
-                    )
+                    this_question_type = question_type or QU_QUESTION if first_request else QM_QUESTION
+                    out = self._generate_request_query(zc, now, this_question_type)
                     first_request = False
-                    if not out.questions:
-                        return self.load_from_cache(zc, now)
-                    zc.async_send(out, addr, port)
+                    if out.questions:
+                        # All questions may have been suppressed
+                        # by the question history, so nothing to send,
+                        # but keep waiting for answers in case another
+                        # client on the network is asking the same
+                        # question or they have not arrived yet.
+                        zc.async_send(out, addr, port)
                     next_ = now + delay
-                    delay *= 2
-                    next_ += random.randint(*_AVOID_SYNC_DELAY_RANDOM_INTERVAL)
+                    next_ += self._get_random_delay()
+                    if this_question_type is QM_QUESTION and delay < _DUPLICATE_QUESTION_INTERVAL:
+                        # If we just asked a QM question, we need to
+                        # wait at least the duplicate question interval
+                        # before asking another QM question otherwise
+                        # its likely to be suppressed by the question
+                        # history of the remote responder.
+                        delay = _DUPLICATE_QUESTION_INTERVAL
 
-                await self.async_wait(min(next_, last) - now)
+                await self.async_wait(min(next_, last) - now, zc.loop)
                 now = current_time_millis()
         finally:
             zc.async_remove_listener(self)
 
         return True
 
-    def generate_request_query(
-        self, zc: 'Zeroconf', now: float, question_type: Optional[DNSQuestionType] = None
+    def _add_question_with_known_answers(
+        self,
+        out: DNSOutgoing,
+        qu_question: bool,
+        question_history: QuestionHistory,
+        cache: DNSCache,
+        now: float_,
+        name: str_,
+        type_: int_,
+        class_: int_,
+        skip_if_known_answers: bool,
+    ) -> None:
+        """Add a question with known answers if its not suppressed."""
+        known_answers = {
+            answer for answer in cache.get_all_by_details(name, type_, class_) if not answer.is_stale(now)
+        }
+        if skip_if_known_answers and known_answers:
+            return
+        question = DNSQuestion(name, type_, class_)
+        if qu_question:
+            question.unicast = True
+        elif question_history.suppresses(question, now, known_answers):
+            return
+        else:
+            question_history.add_question_at_time(question, now, known_answers)
+        out.add_question(question)
+        for answer in known_answers:
+            out.add_answer_at_time(answer, now)
+
+    def _generate_request_query(
+        self, zc: 'Zeroconf', now: float_, question_type: DNSQuestionType
     ) -> DNSOutgoing:
         """Generate the request query."""
         out = DNSOutgoing(_FLAGS_QR_QUERY)
         name = self._name
-        server_or_name = self.server or name
+        server = self.server or name
         cache = zc.cache
-        out.add_question_or_one_cache(cache, now, name, _TYPE_SRV, _CLASS_IN)
-        out.add_question_or_one_cache(cache, now, name, _TYPE_TXT, _CLASS_IN)
-        out.add_question_or_all_cache(cache, now, server_or_name, _TYPE_A, _CLASS_IN)
-        out.add_question_or_all_cache(cache, now, server_or_name, _TYPE_AAAA, _CLASS_IN)
-        if question_type == DNSQuestionType.QU:
-            for question in out.questions:
-                question.unicast = True
+        history = zc.question_history
+        qu_question = question_type is QU_QUESTION
+        self._add_question_with_known_answers(
+            out, qu_question, history, cache, now, name, _TYPE_SRV, _CLASS_IN, True
+        )
+        self._add_question_with_known_answers(
+            out, qu_question, history, cache, now, name, _TYPE_TXT, _CLASS_IN, True
+        )
+        self._add_question_with_known_answers(
+            out, qu_question, history, cache, now, server, _TYPE_A, _CLASS_IN, False
+        )
+        self._add_question_with_known_answers(
+            out, qu_question, history, cache, now, server, _TYPE_AAAA, _CLASS_IN, False
+        )
         return out
 
-    def __eq__(self, other: object) -> bool:
-        """Tests equality of service name"""
-        return isinstance(other, ServiceInfo) and other._name == self._name
-
     def __repr__(self) -> str:
         """String representation"""
         return '{}({})'.format(
