@@ -4,11 +4,12 @@
 """ conftest for zeroconf tests. """
 
 import threading
-import unittest
+from unittest.mock import patch
 
 import pytest
 
 from zeroconf import _core, const
+from zeroconf._handlers import query_handler
 
 
 @pytest.fixture(autouse=True)
@@ -23,9 +24,9 @@ def verify_threads_ended():
 @pytest.fixture
 def run_isolated():
     """Change the mDNS port to run the test in isolation."""
-    with unittest.mock.patch.object(_core, "_MDNS_PORT", 5454), unittest.mock.patch.object(
-        const, "_MDNS_PORT", 5454
-    ):
+    with patch.object(query_handler, "_MDNS_PORT", 5454), patch.object(
+        _core, "_MDNS_PORT", 5454
+    ), patch.object(const, "_MDNS_PORT", 5454):
         yield
 
 
@@ -36,7 +37,5 @@ def disable_duplicate_packet_suppression():
     Some tests run too slowly because of the duplicate
     packet suppression.
     """
-    with unittest.mock.patch.object(
-        _core, "_DUPLICATE_PACKET_SUPPRESSION_INTERVAL", 0
-    ), unittest.mock.patch.object(const, "_DUPLICATE_PACKET_SUPPRESSION_INTERVAL", 0):
+    with patch.object(const, "_DUPLICATE_PACKET_SUPPRESSION_INTERVAL", 0):
         yield
